@@ -37,6 +37,14 @@ options:
             - CIDR defining the IPv4 address space of the subnet. Must be valid within the context of the virtual network.
         aliases:
             - address_prefix
+    address_prefixes_cidr:
+        description:
+            - CIDR defining the IPv4 and IPv6 address space of the subnet. Must be valid within the context of the virtual network.
+            - If set I(address_prefix), It will not set.
+        aliases:
+            - address_prefixes
+        type: list
+        version_added: "1.0.0"
     security_group:
         description:
             - Existing security group with which to associate the subnet.
@@ -76,6 +84,22 @@ options:
                 description:
                     - A list of locations.
                 type: list
+    private_endpoint_network_policies:
+        description:
+            - C(Enabled) or C(Disabled) apply network policies on private endpoints in the subnet.
+        type: str
+        default: Enabled
+        choices:
+            - Enabled
+            - Disabled
+    private_link_service_network_policies:
+        description:
+            - C(Enabled) or C(Disabled) apply network policies on private link service in the subnet.
+        type: str
+        default: Enabled
+        choices:
+            - Enabled
+            - Disabled
 
 extends_documentation_fragment:
     - azure.azcollection.azure
@@ -126,6 +150,12 @@ state:
             returned: always
             type: str
             sample: "10.1.0.0/16"
+        address_prefixes:
+            description:
+                - IP address for IPv4 and IPv6 CIDR.
+            returned: always
+            type: list
+            sample: ["10.2.0.0/24", "fdda:e69b:1587:495e::/64"]
         id:
             description:
                 - Subnet resource path.
@@ -162,6 +192,18 @@ state:
             returned: always
             type: str
             sample: "Succeeded"
+        private_endpoint_network_policies:
+            description:
+                - C(Enabled) or C(Disabled) apply network policies on private endpoints in the subnet.
+            returned: always
+            type: str
+            sample: "Enabled"
+        private_link_service_network_policies:
+            description:
+                - C(Enabled) or C(Disabled) apply network policies on private link service in the subnet.
+            returned: always
+            type: str
+            sample: "Disabled"
 '''  # NOQA
 
 from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase, CIDR_PATTERN, azure_id_to_dict, format_resource_id
@@ -179,8 +221,11 @@ def subnet_to_dict(subnet):
         name=subnet.name,
         provisioning_state=subnet.provisioning_state,
         address_prefix=subnet.address_prefix,
+        address_prefixes=subnet.address_prefixes,
         network_security_group=dict(),
-        route_table=dict()
+        route_table=dict(),
+        private_endpoint_network_policies=subnet.private_endpoint_network_policies,
+        private_link_service_network_policies=subnet.private_link_service_network_policies
     )
     if subnet.network_security_group:
         id_keys = azure_id_to_dict(subnet.network_security_group.id)
@@ -207,13 +252,26 @@ class AzureRMSubnet(AzureRMModuleBase):
             state=dict(type='str', default='present', choices=['present', 'absent']),
             virtual_network_name=dict(type='str', required=True, aliases=['virtual_network']),
             address_prefix_cidr=dict(type='str', aliases=['address_prefix']),
+            address_prefixes_cidr=dict(type='list', aliases=['address_prefixes']),
             security_group=dict(type='raw', aliases=['security_group_name']),
             route_table=dict(type='raw'),
             service_endpoints=dict(
                 type='list'
+            ),
+            private_endpoint_network_policies=dict(
+                type='str',
+                default='Enabled',
+                choices=['Enabled', 'Disabled']
+            ),
+            private_link_service_network_policies=dict(
+                type='str',
+                default='Enabled',
+                choices=['Enabled', 'Disabled']
             )
         )
 
+        mutually_exclusive = [['address_prefix_cidr', 'address_prefixes_cidr']]
+
         self.results = dict(
             changed=False,
             state=dict()
@@ -224,9 +282,12 @@ class AzureRMSubnet(AzureRMModuleBase):
         self.state = None
         self.virtual_network_name = None
         self.address_prefix_cidr = None
+        self.address_prefixes_cidr = None
         self.security_group = None
         self.route_table = None
         self.service_endpoints = None
+        self.private_link_service_network_policies = None
+        self.private_endpoint_network_policies = None
 
         super(AzureRMSubnet, self).__init__(self.module_arg_spec,
                                             supports_check_mode=True,
@@ -268,10 +329,29 @@ class AzureRMSubnet(AzureRMModuleBase):
             results = subnet_to_dict(subnet)
 
             if self.state == 'present':
+                if self.private_endpoint_network_policies is not None:
+                    if results['private_endpoint_network_policies'] != self.private_endpoint_network_policies:
+                        self.log("CHANGED: subnet {0} private_endpoint_network_policies".format(self.private_endpoint_network_policies))
+                        changed = True
+                        results['private_endpoint_network_policies'] = self.private_endpoint_network_policies
+                else:
+                    subnet['private_endpoint_network_policies'] = results['private_endpoint_network_policies']
+                if self.private_link_service_network_policies is not None:
+                    if results['private_link_service_network_policies'] != self.private_link_service_network_policies is not None:
+                        self.log("CHANGED: subnet {0} private_link_service_network_policies".format(self.private_link_service_network_policies))
+                        changed = True
+                        results['private_link_service_network_policies'] = self.private_link_service_network_policies
+                else:
+                    subnet['private_link_service_network_policies'] = results['private_link_service_network_policies']
+
                 if self.address_prefix_cidr and results['address_prefix'] != self.address_prefix_cidr:
                     self.log("CHANGED: subnet {0} address_prefix_cidr".format(self.name))
                     changed = True
                     results['address_prefix'] = self.address_prefix_cidr
+                if self.address_prefixes_cidr and results['address_prefixes'] != self.address_prefixes_cidr:
+                    self.log("CHANGED: subnet {0} address_prefixes_cidr".format(self.name))
+                    changed = True
+                    results['address_prefixes'] = self.address_prefixes_cidr
 
                 if self.security_group is not None and results['network_security_group'].get('id') != nsg.get('id'):
                     self.log("CHANGED: subnet {0} network security group".format(self.name))
@@ -314,11 +394,12 @@ class AzureRMSubnet(AzureRMModuleBase):
             if self.state == 'present' and changed:
                 if not subnet:
                     # create new subnet
-                    if not self.address_prefix_cidr:
-                        self.fail('address_prefix_cidr is not set')
+                    if not self.address_prefix_cidr and not self.address_prefixes_cidr:
+                        self.fail('address_prefix_cidr or address_prefixes_cidr is not set')
                     self.log('Creating subnet {0}'.format(self.name))
                     subnet = self.network_models.Subnet(
-                        address_prefix=self.address_prefix_cidr
+                        address_prefix=self.address_prefix_cidr,
+                        address_prefixes=self.address_prefixes_cidr
                     )
                     if nsg:
                         subnet.network_security_group = self.network_models.NetworkSecurityGroup(id=nsg.get('id'))
@@ -326,11 +407,16 @@ class AzureRMSubnet(AzureRMModuleBase):
                         subnet.route_table = self.network_models.RouteTable(id=self.route_table)
                     if self.service_endpoints:
                         subnet.service_endpoints = self.service_endpoints
+                    if self.private_endpoint_network_policies:
+                        subnet.private_endpoint_network_policies = self.private_endpoint_network_policies
+                    if self.private_link_service_network_policies:
+                        subnet.private_link_service_network_policies = self.private_link_service_network_policies
                 else:
                     # update subnet
                     self.log('Updating subnet {0}'.format(self.name))
                     subnet = self.network_models.Subnet(
-                        address_prefix=results['address_prefix']
+                        address_prefix=results['address_prefix'],
+                        address_prefixes=results['address_prefixes']
                     )
                     if results['network_security_group'].get('id') is not None:
                         subnet.network_security_group = self.network_models.NetworkSecurityGroup(id=results['network_security_group'].get('id'))
@@ -339,6 +425,10 @@ class AzureRMSubnet(AzureRMModuleBase):
 
                     if results.get('service_endpoints') is not None:
                         subnet.service_endpoints = results['service_endpoints']
+                    if results.get('private_link_service_network_policies') is not None:
+                        subnet.private_link_service_network_policies = results['private_link_service_network_policies']
+                    if results.get('private_endpoint_network_policies') is not None:
+                        subnet.private_endpoint_network_policies = results['private_endpoint_network_policies']
 
                 self.results['state'] = self.create_or_update_subnet(subnet)
             elif self.state == 'absent' and changed:
