@@ -59,6 +59,9 @@ from ansible_collections.cisco.ios.plugins.module_utils.network.ios.facts.ospfv2
 from ansible_collections.cisco.ios.plugins.module_utils.network.ios.facts.ospfv3.ospfv3 import (
     Ospfv3Facts,
 )
+from ansible_collections.cisco.ios.plugins.module_utils.network.ios.facts.ospf_interfaces.ospf_interfaces import (
+    Ospf_InterfacesFacts,
+)
 from ansible_collections.cisco.ios.plugins.module_utils.network.ios.facts.legacy.base import (
     Default,
     Hardware,
@@ -86,6 +89,7 @@ FACT_RESOURCE_SUBSETS = dict(
     acls=AclsFacts,
     ospfv2=Ospfv2Facts,
     ospfv3=Ospfv3Facts,
+    ospf_interfaces=Ospf_InterfacesFacts,
 )
 
 
