@@ -9,7 +9,7 @@ __metaclass__ = type
 DOCUMENTATION = """
   lookup: hashi_vault
   author:
-    - Jonathan Davila <jdavila(at)ansible.com>
+    - Jonathan Davila (!UNKNOWN) <jdavila(at)ansible.com>
     - Brian Scholer (@briantist)
   short_description: Retrieve secrets from HashiCorp's vault
   requirements:
@@ -38,13 +38,17 @@ DOCUMENTATION = """
     token_path:
       description: If no token is specified, will try to read the token file from this path.
       env:
-        - name: HOME
+        - name: VAULT_TOKEN_PATH
+          version_added: 1.2.0
       ini:
         - section: lookup_hashi_vault
           key: token_path
       version_added: '0.2.0'
     token_file:
       description: If no token is specified, will try to read the token from this file in C(token_path).
+      env:
+        - name: VAULT_TOKEN_FILE
+          version_added: 1.2.0
       ini:
         - section: lookup_hashi_vault
           key: token_file
@@ -117,6 +121,9 @@ DOCUMENTATION = """
       default: True
     namespace:
       description: Namespace where secrets reside. Requires HVAC 0.7.0+ and Vault 0.11+.
+      env:
+        - name: VAULT_NAMESPACE
+          version_added: 1.2.0
     aws_profile:
         description: The AWS profile
         type: str
@@ -241,6 +248,8 @@ RETURN = """
 _raw:
   description:
     - secrets(s) requested
+  type: list
+  elements: dict
 """
 
 import os
@@ -404,7 +413,7 @@ class HashiVault:
             self.client.auth_ldap(**params)
 
     def auth_approle(self):
-        params = self.get_options('role_id', 'secret_id')
+        params = self.get_options('role_id', 'secret_id', 'mount_point')
         self.client.auth_approle(**params)
 
     def auth_aws_iam_login(self):
@@ -532,6 +541,11 @@ class LookupModule(LookupBase):
 
     def validate_auth_token(self, auth_method):
         if auth_method == 'token':
+            if not self.get_option('token_path'):
+                # generally we want env vars defined in the spec, but in this case we want
+                # the env var HOME to have lower precedence than any other value source,
+                # including ini, so we're doing it here after all other processing has taken place
+                self.set_option('token_path', os.environ.get('HOME'))
             if not self.get_option('token') and self.get_option('token_path'):
                 token_filename = os.path.join(
                     self.get_option('token_path'),
