@@ -31,6 +31,9 @@
 # Contains utility methods
 # Lenovo Networking
 
+from __future__ import absolute_import, division, print_function
+__metaclass__ = type
+
 import time
 import socket
 import re
