@@ -19,7 +19,7 @@ module: na_ontap_export_policy_rule
 short_description: NetApp ONTAP manage export policy rules
 extends_documentation_fragment:
     - netapp.ontap.netapp.na_ontap
-version_added: '2.6'
+version_added: 2.6.0
 author: NetApp Ansible Team (@carchi8py) <ng-ansibleteam@netapp.com>
 
 description:
@@ -153,7 +153,6 @@ RETURN = """
 """
 import traceback
 
-import json
 from ansible.module_utils.basic import AnsibleModule
 from ansible.module_utils._text import to_native
 import ansible_collections.netapp.ontap.plugins.module_utils.netapp as netapp_utils
