@@ -16,7 +16,7 @@ module: na_ontap_net_routes
 short_description: NetApp ONTAP network routes
 extends_documentation_fragment:
     - netapp.ontap.netapp.na_ontap
-version_added: '2.6'
+version_added: 2.6.0
 author: NetApp Ansible Team (@carchi8py) <ng-ansibleteam@netapp.com>
 description:
 - Modify ONTAP network routes.
@@ -243,7 +243,7 @@ class NetAppOntapNetRoutes(object):
                     value = current[attribute]
                 route_obj.add_new_child(attribute, str(value))
             try:
-                result = self.server.invoke_successfully(route_obj, True)
+                self.server.invoke_successfully(route_obj, True)
             except netapp_utils.zapi.NaApiError as error:
                 # restore the old route, create the route with the existing metric
                 self.create_net_route(current['metric'])
