@@ -19,7 +19,7 @@ module: na_ontap_qtree
 short_description: NetApp ONTAP manage qtrees
 extends_documentation_fragment:
     - netapp.ontap.netapp.na_ontap
-version_added: '2.6'
+version_added: 2.6.0
 author: NetApp Ansible Team (@carchi8py) <ng-ansibleteam@netapp.com>
 
 description:
@@ -48,7 +48,8 @@ options:
 
   flexvol_name:
     description:
-    - The name of the FlexVol the qtree should exist on. Required when C(state=present).
+    - The name of the FlexVol the qtree should exist on.
+    required: true
     type: str
 
   vserver:
@@ -92,6 +93,25 @@ options:
     default: true
     version_added: 20.8.0
 
+  wait_for_completion:
+    description:
+      - Only applicable for REST.  When using ZAPI, the deletion is always synchronous.
+      - Deleting a qtree may take time if many files need to be deleted.
+      - Set this parameter to 'true' for synchronous execution during delete.
+      - Set this parameter to 'false' for asynchronous execution.
+      - For asynchronous, execution exits as soon as the request is sent, and the qtree is deleted in background.
+    type: bool
+    default: true
+    version_added: 2.9.0
+
+  time_out:
+    description:
+      - Maximum time to wait for qtree deletion in seconds when wait_for_completion is True.
+      - Error out if task is not completed in defined time.
+      - Default is set to 3 minutes.
+    default: 180
+    type: int
+    version_added: 2.9.0
 '''
 
 EXAMPLES = """
@@ -145,13 +165,15 @@ class NetAppOntapQTree(object):
             state=dict(required=False, type='str', choices=['present', 'absent'], default='present'),
             name=dict(required=True, type='str'),
             from_name=dict(required=False, type='str'),
-            flexvol_name=dict(required=False, type='str'),
+            flexvol_name=dict(required=True, type='str'),
             vserver=dict(required=True, type='str'),
             export_policy=dict(required=False, type='str'),
             security_style=dict(required=False, type='str', choices=['unix', 'ntfs', 'mixed']),
             oplocks=dict(required=False, type='str', choices=['enabled', 'disabled']),
             unix_permissions=dict(required=False, type='str'),
-            force_delete=dict(required=False, type='bool', default=True)
+            force_delete=dict(required=False, type='bool', default=True),
+            wait_for_completion=dict(required=False, type='bool', default=True),
+            time_out=dict(required=False, type='int', default=180),
         ))
 
         self.module = AnsibleModule(
@@ -164,8 +186,8 @@ class NetAppOntapQTree(object):
         self.na_helper = NetAppModule()
         self.parameters = self.na_helper.set_parameters(self.module.params)
 
-        self.restApi = OntapRestAPI(self.module)
-        if self.restApi.is_rest():
+        self.rest_api = OntapRestAPI(self.module)
+        if self.rest_api.is_rest():
             self.use_rest = True
         else:
             if HAS_NETAPP_LIB is False:
@@ -189,11 +211,11 @@ class NetAppOntapQTree(object):
             name = self.parameters['name']
         if self.use_rest:
             api = "storage/qtrees"
-            params = {'fields': 'export_policy,unix_permissions,security_style,volume',
-                      'svm.name': self.parameters['vserver'],
-                      'volume': self.parameters['flexvol_name'],
-                      'name': name}
-            message, error = self.restApi.get(api, params)
+            query = {'fields': 'export_policy,unix_permissions,security_style,volume',
+                     'svm.name': self.parameters['vserver'],
+                     'volume': self.parameters['flexvol_name'],
+                     'name': name}
+            message, error = self.rest_api.get(api, query)
             if error:
                 self.module.fail_json(msg=error)
             if len(message.keys()) == 0:
@@ -216,8 +238,7 @@ class NetAppOntapQTree(object):
             result = self.server.invoke_successfully(qtree_list_iter,
                                                      enable_tunneling=True)
             return_q = None
-            if (result.get_child_by_name('num-records') and
-                    int(result.get_child_content('num-records')) >= 1):
+            if (result.get_child_by_name('num-records') and int(result.get_child_content('num-records')) >= 1):
                 return_q = {'export_policy': result['attributes-list']['qtree-info']['export-policy'],
                             'oplocks': result['attributes-list']['qtree-info']['oplocks'],
                             'security_style': result['attributes-list']['qtree-info']['security-style']}
@@ -235,15 +256,15 @@ class NetAppOntapQTree(object):
         """
         if self.use_rest:
             api = "storage/qtrees"
-            params = {'name': self.parameters['name'], 'volume': {'name': self.parameters['flexvol_name']},
-                      'svm': {'name': self.parameters['vserver']}}
+            body = {'name': self.parameters['name'], 'volume': {'name': self.parameters['flexvol_name']},
+                    'svm': {'name': self.parameters['vserver']}}
             if self.parameters.get('export_policy'):
-                params['export_policy'] = self.parameters['export_policy']
+                body['export_policy'] = self.parameters['export_policy']
             if self.parameters.get('security_style'):
-                params['security_style'] = self.parameters['security_style']
+                body['security_style'] = self.parameters['security_style']
             if self.parameters.get('unix_permissions'):
-                params['unix_permissions'] = self.parameters['unix_permissions']
-            __, error = self.restApi.post(api, params)
+                body['unix_permissions'] = self.parameters['unix_permissions']
+            __, error = self.rest_api.post(api, body)
             if error:
                 self.module.fail_json(msg=error)
         else:
@@ -272,12 +293,16 @@ class NetAppOntapQTree(object):
         """
         if self.use_rest:
             uuid = current['volume']['uuid']
-            id = str(current['id'])
-            api = "storage/qtrees/%s/%s" % (uuid, id)
-            params = None
-            dummy, error = self.restApi.delete(api, params)
+            qid = str(current['id'])
+            api = "storage/qtrees/%s/%s" % (uuid, qid)
+            body = None
+            query = {'return_timeout': 3}
+            response, error = self.rest_api.delete(api, body, query)
             if error:
                 self.module.fail_json(msg=error)
+            if 'job' in response and self.parameters['wait_for_completion']:
+                self.rest_api.wait_on_job(response['job'], timeout=self.parameters['time_out'], increment=10)
+
         else:
             path = '/vol/%s/%s' % (self.parameters['flexvol_name'], self.parameters['name'])
             options = {'qtree': path}
@@ -293,18 +318,16 @@ class NetAppOntapQTree(object):
                 self.module.fail_json(msg="Error deleting qtree %s: %s" % (path, to_native(error)),
                                       exception=traceback.format_exc())
 
-    def rename_qtree(self, current=None):
+    def rename_qtree(self, current):
         """
         Rename a qtree
         """
         if self.use_rest:
-            if current is None:
-                current = self.get_qtree(self.parameters['from_name'])
-            params = {'name': self.parameters['name']}
+            body = {'name': self.parameters['name']}
             uuid = current['volume']['uuid']
-            id = str(current['id'])
-            api = "storage/qtrees/%s/%s" % (uuid, id)
-            dummy, error = self.restApi.patch(api, params)
+            qid = str(current['id'])
+            api = "storage/qtrees/%s/%s" % (uuid, qid)
+            dummy, error = self.rest_api.patch(api, body)
             if error:
                 self.module.fail_json(msg=error)
         else:
@@ -322,27 +345,25 @@ class NetAppOntapQTree(object):
                                       % (self.parameters['from_name'], to_native(error)),
                                       exception=traceback.format_exc())
 
-    def modify_qtree(self, current=None):
+    def modify_qtree(self, current):
         """
         Modify a qtree
         """
         if self.use_rest:
             now = datetime.datetime.now()
-            if current is None:
-                current = self.get_qtree(self.parameters['name'])
-            params = {}
+            body = {}
             if self.parameters.get('security_style'):
-                params['security_style'] = self.parameters['security_style']
+                body['security_style'] = self.parameters['security_style']
             if self.parameters.get('unix_permissions'):
-                params['unix_permissions'] = self.parameters['unix_permissions']
+                body['unix_permissions'] = self.parameters['unix_permissions']
             if self.parameters.get('export_policy'):
-                params['export_policy'] = {'name': self.parameters['export_policy']}
+                body['export_policy'] = {'name': self.parameters['export_policy']}
             uuid = current['volume']['uuid']
-            id = str(current['id'])
-            api = "storage/qtrees/%s/%s" % (uuid, id)
+            qid = str(current['id'])
+            api = "storage/qtrees/%s/%s" % (uuid, qid)
             timeout = 120
-            data = {'return_timeout': timeout}
-            dummy, error = self.restApi.patch(api, params, data)
+            query = {'return_timeout': timeout}
+            dummy, error = self.rest_api.patch(api, body, query)
 
             later = datetime.datetime.now()
             time_elapsed = later - now
@@ -377,7 +398,12 @@ class NetAppOntapQTree(object):
         current = self.get_qtree()
         rename, cd_action, modify = None, None, None
         if self.parameters.get('from_name'):
-            rename = self.na_helper.is_rename_action(self.get_qtree(self.parameters['from_name']), current)
+            from_qtree = self.get_qtree(self.parameters['from_name'])
+            rename = self.na_helper.is_rename_action(from_qtree, current)
+            if rename is None:
+                self.module.fail_json(msg='Error renaming: qtree %s does not exist' % self.parameters['from_name'])
+            if rename:
+                current = from_qtree
         else:
             cd_action = self.na_helper.get_cd_action(current, self.parameters)
         if cd_action is None and self.parameters['state'] == 'present':
@@ -404,14 +430,15 @@ class NetAppOntapQTree(object):
             if self.module.check_mode:
                 pass
             else:
-                if rename:
-                    self.rename_qtree()
-                elif cd_action == 'create':
+                if cd_action == 'create':
                     self.create_qtree()
                 elif cd_action == 'delete':
                     self.delete_qtree(current)
-                elif modify:
-                    self.modify_qtree(current)
+                else:
+                    if rename:
+                        self.rename_qtree(current)
+                    if modify:
+                        self.modify_qtree(current)
         self.module.exit_json(changed=self.na_helper.changed)
 
 
