@@ -22,6 +22,7 @@ extends_documentation_fragment:
 short_description: NetApp ONTAP information gatherer using REST APIs
 description:
     - This module allows you to gather various information about ONTAP configuration using REST APIs
+version_added: 20.5.0
 
 options:
     state:
@@ -38,6 +39,10 @@ options:
                 to a given subset.  Either the info name or the Rest API can be given.
                 Possible values for this argument include
                 "aggregate_info" or "storage/aggregates",
+                "application_info" or "application/applications",
+                "application_template_info" or "application/templates",
+                "autosupport_config_info" or "support/autosupport",
+                "autosupport_messages_history" or "support/autosupport/messages",
                 "broadcast_domains_info" or "network/ethernet/broadcast-domains",
                 "cifs_services_info" or "protocols/cifs/services",
                 "cifs_share_info" or "protocols/cifs/shares",
@@ -51,13 +56,33 @@ options:
                 "cluster_software_history" or "cluster/software/history",
                 "cluster_software_packages" or "cluster/software/packages",
                 "disk_info" or "storage/disks",
+                "initiator_groups_info" or "protocols/san/igroups",
                 "ip_interfaces_info" or "network/ip/interfaces",
                 "ip_routes_info" or "network/ip/routes",
                 "ip_service_policies" or "network/ip/service-policies",
                 "network_ipspaces_info" or "network/ipspaces",
                 "network_ports_info" or "network/ethernet/ports",
+                "ontap_system_version" or "cluster/software",
                 "san_fc_logins_info" or "network/fc/logins",
                 "san_fc_wppn-aliases" or "network/fc/wwpn-aliases",
+                "san_fcp_services" or "protocols/san/fcp/services",
+                "san_iscsi_credentials" or "protocols/san/iscsi/credentials",
+                "san_iscsi_services" or "protocols/san/iscsi/services",
+                "san_lun_maps" or "protocols/san/lun-maps",
+                "storage_flexcaches_info" or "storage/flexcache/flexcaches",
+                "storage_flexcaches_origin_info" or "storage/flexcache/origins",
+                "storage_luns_info" or "storage/luns",
+                "storage_NVMe_namespaces" or "storage/namespaces",
+                "storage_ports_info" or "storage/ports",
+                "storage_qos_policies" or "storage/qos/policies",
+                "storage_qtrees_config" or "storage/qtrees",
+                "storage_quota_reports" or "storage/quota/reports",
+                "storage_quota_policy_rules" or "storage/quota/rules",
+                "storage_shelves_config" or "storage/shelves",
+                "storage_snapshot_policies" or "storage/snapshot-policies",
+                "support_ems_config" or "support/ems",
+                "support_ems_events" or "support/ems/events",
+                "support_ems_filters" or "support/ems/filters",
                 "svm_dns_config_info" or "name-services/dns",
                 "svm_ldap_config_info" or "name-services/ldap",
                 "svm_name_mapping_config_info" or "name-services/name-mappings",
@@ -265,6 +290,10 @@ class NetAppONTAPGatherInfo(object):
         """
         info_to_rest_mapping = {
             "aggregate_info": "storage/aggregates",
+            "application_info": "application/applications",
+            "application_template_info": "application/templates",
+            "autosupport_config_info": "support/autosupport",
+            "autosupport_messages_history": "support/autosupport/messages",
             "broadcast_domains_info": "network/ethernet/broadcast-domains",
             "cifs_services_info": "protocols/cifs/services",
             "cifs_share_info": "protocols/cifs/shares",
@@ -279,13 +308,33 @@ class NetAppONTAPGatherInfo(object):
             "cluster_software_history": "cluster/software/history",
             "cluster_software_packages": "cluster/software/packages",
             "disk_info": "storage/disks",
+            "initiator_groups_info": "protocols/san/igroups",
             "ip_interfaces_info": "network/ip/interfaces",
             "ip_routes_info": "network/ip/routes",
             "ip_service_policies": "network/ip/service-policies",
             "network_ipspaces_info": "network/ipspaces",
             "network_ports_info": "network/ethernet/ports",
+            "ontap_system_version": "cluster/software",
             "san_fc_logins_info": "network/fc/logins",
             "san_fc_wppn-aliases": "network/fc/wwpn-aliases",
+            "san_fcp_services": "protocols/san/fcp/services",
+            "san_iscsi_credentials": "protocols/san/iscsi/credentials",
+            "san_iscsi_services": "protocols/san/iscsi/services",
+            "san_lun_maps": "protocols/san/lun-maps",
+            "storage_flexcaches_info": "storage/flexcache/flexcaches",
+            "storage_flexcaches_origin_info": "storage/flexcache/origins",
+            "storage_luns_info": "storage/luns",
+            "storage_NVMe_namespaces": "storage/namespaces",
+            "storage_ports_info": "storage/ports",
+            "storage_qos_policies": "storage/qos/policies",
+            "storage_qtrees_config": "storage/qtrees",
+            "storage_quota_reports": "storage/quota/reports",
+            "storage_quota_policy_rules": "storage/quota/rules",
+            "storage_shelves_config": "storage/shelves",
+            "storage_snapshot_policies": "storage/snapshot-policies",
+            "support_ems_config": "support/ems",
+            "support_ems_events": "support/ems/events",
+            "support_ems_filters": "support/ems/filters",
             "svm_dns_config_info": "name-services/dns",
             "svm_ldap_config_info": "name-services/ldap",
             "svm_name_mapping_config_info": "name-services/name-mappings",
@@ -318,6 +367,12 @@ class NetAppONTAPGatherInfo(object):
 
         # Defining gather_subset and appropriate api_call
         get_ontap_subset_info = {
+            'application/applications': {
+                'api_call': 'application/applications',
+            },
+            'application/templates': {
+                'api_call': 'application/templates',
+            },
             'cloud/targets': {
                 'api_call': 'cloud/targets',
             },
@@ -343,6 +398,9 @@ class NetAppONTAPGatherInfo(object):
             'cluster/schedules': {
                 'api_call': 'cluster/schedules',
             },
+            'cluster/software': {
+                'api_call': 'cluster/software',
+            },
             'cluster/software/history': {
                 'api_call': 'cluster/software/history',
             },
@@ -385,15 +443,84 @@ class NetAppONTAPGatherInfo(object):
             'network/ipspaces': {
                 'api_call': 'network/ipspaces',
             },
+            'protocols/cifs/services': {
+                'api_call': 'protocols/cifs/services',
+            },
+            'protocols/cifs/shares': {
+                'api_call': 'protocols/cifs/shares',
+            },
+            'protocols/san/fcp/services': {
+                'api_call': 'protocols/san/fcp/services',
+            },
+            'protocols/san/igroups': {
+                'api_call': 'protocols/san/igroups',
+            },
+            'protocols/san/iscsi/credentials': {
+                'api_call': 'protocols/san/iscsi/credentials',
+            },
+            'protocols/san/iscsi/services': {
+                'api_call': 'protocols/san/iscsi/services',
+            },
+            'protocols/san/lun-maps': {
+                'api_call': 'protocols/san/lun-maps',
+            },
             'storage/aggregates': {
                 'api_call': 'storage/aggregates',
             },
             'storage/disks': {
                 'api_call': 'storage/disks',
             },
+            'storage/flexcache/flexcaches': {
+                'api_call': 'storage/flexcache/flexcaches',
+            },
+            'storage/flexcache/origins': {
+                'api_call': 'storage/flexcache/origins',
+            },
+            'storage/luns': {
+                'api_call': 'storage/luns',
+            },
+            'storage/namespaces': {
+                'api_call': 'storage/namespaces',
+            },
+            'storage/ports': {
+                'api_call': 'storage/ports',
+            },
+            'storage/qos/policies': {
+                'api_call': 'storage/qos/policies',
+            },
+            'storage/qtrees': {
+                'api_call': 'storage/qtrees',
+            },
+            'storage/quota/reports': {
+                'api_call': 'storage/quota/reports',
+            },
+            'storage/quota/rules': {
+                'api_call': 'storage/quota/rules',
+            },
+            'storage/shelves': {
+                'api_call': 'storage/shelves',
+            },
+            'storage/snapshot-policies': {
+                'api_call': 'storage/snapshot-policies',
+            },
             'storage/volumes': {
                 'api_call': 'storage/volumes',
             },
+            'support/autosupport': {
+                'api_call': 'support/autosupport',
+            },
+            'support/autosupport/messages': {
+                'api_call': 'support/autosupport/messages',
+            },
+            'support/ems': {
+                'api_call': 'support/ems',
+            },
+            'support/ems/events': {
+                'api_call': 'support/ems/events',
+            },
+            'support/ems/filters': {
+                'api_call': 'support/ems/filters',
+            },
             'svm/peers': {
                 'api_call': 'svm/peers',
             },
@@ -402,12 +529,6 @@ class NetAppONTAPGatherInfo(object):
             },
             'svm/svms': {
                 'api_call': 'svm/svms',
-            },
-            'protocols/cifs/services': {
-                'api_call': 'protocols/cifs/services',
-            },
-            'protocols/cifs/shares': {
-                'api_call': 'protocols/cifs/shares',
             }
         }
 
