@@ -35,6 +35,12 @@ DOCUMENTATION = """
                 - Allows connection when SSL certificates are not valid. Set to C(false) when certificates are not trusted.
             default: True
             type: boolean
+        follow_redirects:
+            description:
+                - Determine how redirects are followed.
+                - By default, I(follow_redirects) is set to uses urllib2 default behavior.
+            default: urllib2
+            choices: ['urllib2', 'all', 'yes', 'safe', 'none']
         config_context:
             description:
                 - If True, it adds config_context in host vars.
@@ -264,12 +270,24 @@ class InventoryModule(BaseInventoryPlugin, Constructable, Cacheable):
                     headers=self.headers,
                     timeout=self.timeout,
                     validate_certs=self.validate_certs,
+                    follow_redirects=self.follow_redirects,
                 )
             except urllib_error.HTTPError as e:
                 """This will return the response body when we encounter an error.
-                   This is to help determine what might be the issue when encountering an error.
-                   Please check issue #294 for more info.
+                This is to help determine what might be the issue when encountering an error.
+                Please check issue #294 for more info.
                 """
+                # Prevent inventory from failing completely if the token does not have the proper permissions for specific URLs
+                if e.code == 403:
+                    self.display.display(
+                        "Permission denied: {0}. This may impair functionality of the inventory plugin.".format(
+                            url
+                        ),
+                        color="red",
+                    )
+                    # Need to return mock response data that is empty to prevent any failures downstream
+                    return {"results": [], "next": None}
+
                 raise AnsibleError(to_native(e.fp.read()))
 
             try:
@@ -292,8 +310,8 @@ class InventoryModule(BaseInventoryPlugin, Constructable, Cacheable):
 
     def get_resource_list(self, api_url):
         """Retrieves resource list from netbox API.
-         Returns:
-            A list of all resource from netbox API.
+        Returns:
+           A list of all resource from netbox API.
         """
         if not api_url:
             raise AnsibleError("Please check API URL in script configuration file.")
@@ -579,7 +597,18 @@ class InventoryModule(BaseInventoryPlugin, Constructable, Cacheable):
             return
 
     def extract_tags(self, host):
-        return host["tags"]
+        try:
+            tag_zero = host["tags"][0]
+            # Check the type of the first element in the "tags" array.
+            # If a dictionary (Netbox >= 2.9), return an array of tags' slugs.
+            if isinstance(tag_zero, dict):
+                return list(sub["slug"] for sub in host["tags"])
+            # If a string (Netbox <= 2.8), return the original "tags" array.
+            elif isinstance(tag_zero, str):
+                return host["tags"]
+        # If tag_zero fails definition (no tags), return the empty array.
+        except Exception:
+            return host["tags"]
 
     def extract_interfaces(self, host):
         try:
@@ -592,11 +621,19 @@ class InventoryModule(BaseInventoryPlugin, Constructable, Cacheable):
 
             interfaces = list(interfaces_lookup[host["id"]].values())
 
+            before_netbox_v29 = bool(self.ipaddresses_lookup)
             # Attach IP Addresses to their interface
             for interface in interfaces:
-                interface["ip_addresses"] = list(
-                    self.ipaddresses_lookup[interface["id"]].values()
-                )
+                if before_netbox_v29:
+                    interface["ip_addresses"] = list(
+                        self.ipaddresses_lookup[interface["id"]].values()
+                    )
+                else:
+                    interface["ip_addresses"] = list(
+                        self.vm_ipaddresses_lookup[interface["id"]].values()
+                        if host["is_virtual"]
+                        else self.device_ipaddresses_lookup[interface["id"]].values()
+                    )
 
             return interfaces
         except Exception:
@@ -900,12 +937,32 @@ class InventoryModule(BaseInventoryPlugin, Constructable, Cacheable):
         # Construct a dictionary of lists, to allow looking up ip addresses by interface id
         # Note that interface ids share the same namespace for both devices and vms so this is a single dictionary
         self.ipaddresses_lookup = defaultdict(dict)
+        # NetBox v2.9 and onwards
+        self.vm_ipaddresses_lookup = defaultdict(dict)
+        self.device_ipaddresses_lookup = defaultdict(dict)
 
         for ipaddress in ipaddresses:
+            # As of NetBox v2.9 "assigned_object_x" replaces "interface"
+            if ipaddress.get("assigned_object_id"):
+                interface_id = ipaddress["assigned_object_id"]
+                ip_id = ipaddress["id"]
+                # We need to copy the ipaddress entry to preserve the original in case caching is used.
+                ipaddress_copy = ipaddress.copy()
+
+                if ipaddress["assigned_object_type"] == "virtualization.vminterface":
+                    self.vm_ipaddresses_lookup[interface_id][ip_id] = ipaddress_copy
+                else:
+                    self.device_ipaddresses_lookup[interface_id][
+                        ip_id
+                    ] = ipaddress_copy  # Remove "assigned_object_X" attributes, as that's redundant when ipaddress is added to an interface
+
+                del ipaddress_copy["assigned_object_id"]
+                del ipaddress_copy["assigned_object_type"]
+                del ipaddress_copy["assigned_object"]
+                continue
 
             if not ipaddress.get("interface"):
                 continue
-
             interface_id = ipaddress["interface"]["id"]
             ip_id = ipaddress["id"]
 
@@ -913,7 +970,6 @@ class InventoryModule(BaseInventoryPlugin, Constructable, Cacheable):
             ipaddress_copy = ipaddress.copy()
 
             self.ipaddresses_lookup[interface_id][ip_id] = ipaddress_copy
-
             # Remove "interface" attribute, as that's redundant when ipaddress is added to an interface
             del ipaddress_copy["interface"]
 
@@ -1357,6 +1413,7 @@ class InventoryModule(BaseInventoryPlugin, Constructable, Cacheable):
         self.timeout = self.get_option("timeout")
         self.max_uri_length = self.get_option("max_uri_length")
         self.validate_certs = self.get_option("validate_certs")
+        self.follow_redirects = self.get_option("follow_redirects")
         self.config_context = self.get_option("config_context")
         self.flatten_config_context = self.get_option("flatten_config_context")
         self.flatten_local_context_data = self.get_option("flatten_local_context_data")
